import logging
from typing import List

from checkov.cloudformation.cfn_utils import get_folder_definitions
from checkov.cloudformation.context_parser import ContextParser
from checkov.cloudformation.graph_builder.local_graph import CloudformationLocalGraph
from checkov.common.graph.graph_manager import GraphManager


class CloudformationGraphManager(GraphManager):
    def __init__(self, db_connector, source='CloudFormation'):
        super().__init__(db_connector=db_connector, parser=None, source=source)

    def build_graph_from_source_directory(self, source_dir,
                                          render_variables=True,
                                          local_graph_class=CloudformationLocalGraph,
                                          parsing_errors=None,
                                          download_external_modules=False,
                                          excluded_paths: List[str] = None):
        logging.info('[CloudformationGraphManager] Parsing files in source dir {source_dir}')
        definitions, definitions_raw = get_folder_definitions(source_dir, excluded_paths)
        if render_variables:
            for cf_file in definitions:
                cf_context_parser = ContextParser(cf_file, definitions[cf_file], definitions_raw[cf_file])
                logging.debug("Template Dump for {}: {}".format(cf_file, definitions[cf_file], indent=2))
                cf_context_parser.evaluate_default_refs()
        logging.info('[CloudformationGraphManager] Building graph from parsed definitions')

        local_graph = local_graph_class(definitions, source=self.source)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph, definitions

    def build_graph_from_definitions(self, definitions, render_variables=False):
        local_graph = CloudformationLocalGraph(definitions, source=self.source)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class CloudfrontDistributionLogging(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Cloudfront distribution has Access Logging enabled"
        id = "CKV_AWS_86"
        supported_resources = ['AWS::CloudFront::Distribution']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/DistributionConfig/Logging/Bucket'

    def get_expected_value(self):
        return ANY_VALUE

check = CloudfrontDistributionLogging()



import os
from typing import List, Dict, Any, Tuple

from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock


def convert_graph_vertices_to_tf_definitions(
    vertices: List[TerraformBlock], root_folder: str
) -> Tuple[Dict[str, Dict[str, Any]], Dict[str, Dict[str, Any]]]:
    tf_definitions: Dict[str, Dict[str, Any]] = {}
    breadcrumbs: Dict[str, Dict[str, Any]] = {}
    for vertex in vertices:
        block_path = vertex.path
        if not os.path.isfile(block_path):
            print(f"tried to convert vertex to tf_definitions but its path doesnt exist: {vertex}")
            continue
        tf_path = block_path
        if vertex.module_dependency:
            tf_path = f"{block_path}[{vertex.module_dependency}#{vertex.module_dependency_num}]"
        block_type = vertex.block_type
        if block_type == BlockType.TF_VARIABLE:
            continue
        tf_definitions.setdefault(tf_path, {}).setdefault(block_type, []).append(vertex.config)
        relative_block_path = f"/{os.path.relpath(block_path, root_folder)}"
        add_breadcrumbs(vertex, breadcrumbs, relative_block_path)
    return tf_definitions, breadcrumbs


def add_breadcrumbs(vertex: TerraformBlock, breadcrumbs: Dict[str, Dict[str, Any]], relative_block_path: str) -> None:
    vertex_breadcrumbs = vertex.breadcrumbs
    if vertex_breadcrumbs:
        breadcrumbs.setdefault(relative_block_path, {})[vertex.name] = vertex_breadcrumbs
